#!/usr/bin/python3
# -*- coding: utf-8 -*-

import serial, time
import threading
import sys, argparse

import findserial

class SerialPort(threading.Thread):
    def __init__(self, serial_port, iterations):
        threading.Thread.__init__(self)
        self.daemon = True
        self.iterations = iterations
        #self.serial_port = serial_port

        self.ser = serial.Serial()
        #ser.port = "/dev/ttyUSB0"
        #ser.port = "/dev/ttyS2"
        self.ser.port = serial_port
        self.ser.baudrate = 9600
        self.ser.bytesize = serial.EIGHTBITS #number of bits per bytes
        self.ser.stopbits = serial.STOPBITS_ONE #number of stop bits
        self.ser.parity = serial.PARITY_NONE #set parity check: no parity
        # ser.timeout = None          #block read
        self.ser.timeout = 1            #non-block read
        #ser.timeout = 2              #timeout block read
        self.ser.xonxoff = False     #disable software flow control
        self.ser.rtscts = False     #disable hardware (RTS/CTS) flow control
        self.ser.dsrdtr = False       #disable hardware (DSR/DTR) flow control
        #self.ser.writeTimeout = 2     #timeout for write

    def run(self):
        try:
            self.ser.open()
            if (self.ser.isOpen()):
                print("Port %s is opened." % self.ser.port)
        except serial.SerialException:
            print("Error with opening serial port %s." % self.ser.port)
            exit()

        if self.ser.isOpen():

            self.ser.flushInput() #flush input buffer, discarding all its contents
            self.ser.flushOutput()#flush output buffer, aborting current output
                                    #and discard all that is in buffer

            time.sleep(0.5)  #give the serial port sometime to receive the data
            numOfLines = 0

            while True:
                try:
                    response = self.ser.readline().decode("utf-8")
                    if response == '':
                        print("%s: unreachable." % self.ser.port)
                    elif response != '\n':
                        if (numOfLines >= self.iterations):
                            break
                        numOfLines += 1
                        print(self.ser.port, end=': ')
                        print(''.join(response.splitlines()))
                except: # just in case
                    print("%s: unreachable." % self.ser.port)

            self.ser.close()

        else:
            print("Cannot open serial port.")

def createParser ():
    parser = argparse.ArgumentParser(
        prog = 'specscan',
        description = 'Wi-fi Spectrum on Pololu Wixel [2403.47–2476.50 MHz with spacing in 286.4 kHz] on CC2511F32',
        epilog = '(c) Oestoidea 2016')
    parser.add_argument ('-i', '--iterations', type = int, default = 3600, help = 'maximum iteration number',
            metavar = 'NUM')
    return parser

if __name__ == '__main__':
    parser = createParser()
    try:
        namespace = parser.parse_args(sys.argv[1:])
    except:
        print('Unknown argument.')

    i = 0
    j = []

    for serial_port in findserial.serial_ports():
        #print(serial_port)
        j.insert(i, SerialPort(serial_port, namespace.iterations))
        j[i].start()
        i += 1

    try:
        j[0].join()
    except:
        print('No connected devices.')

    #if ((threading.active_count() == 1) and i > 0):
    if (i > 0):
        print('%d iterations for %d devices have successfully done.' % (namespace.iterations, i))


"""

#initialization and open the port
#possible timeout values:
#    1. None: wait forever, block call
#    2. 0: non-blocking mode, return immediately
#    3. x, x is bigger than 0, float allowed, timeout block call



try:
    ser.open()
except serial.SerialException:
    print("Error with opening serial port " + str(ser.port))
    exit()

if ser.isOpen():

    ser.flushInput() #flush input buffer, discarding all its contents
    ser.flushOutput()#flush output buffer, aborting current output
    #and discard all that is in buffer
    ser.write("AT+CSQ")
    # print("write data: AT+CSQ")

    time.sleep(0.5)  #give the serial port sometime to receive the data
    numOfLines = 0

    while True:
        response = ser.readline().decode("utf-8")
        if response != '\n':
            print(''.join(response.splitlines()))
            numOfLines = numOfLines + 1
            if (numOfLines >= 10):
                break

    ser.close()

else:
    print("Cannot open serial port ")
"""